'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _nesthydrationjs = require('nesthydrationjs');

var _queryASTToSqlAST = require('./queryASTToSqlAST');

var _stringifySqlAST = require('./stringifySqlAST');

var _stringifySqlAST2 = _interopRequireDefault(_stringifySqlAST);

var _defineObjectShape = require('./defineObjectShape');

var _defineObjectShape2 = _interopRequireDefault(_defineObjectShape);

var _util = require('./util');

var _util2 = require('util');

var _util3 = _interopRequireDefault(_util2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('join-monster');

function joinMonster(ast, context, dbCall) {
  const sqlAST = (0, _queryASTToSqlAST.queryASTToSqlAST)(ast);
  const { sql, shapeDefinition } = compileSqlAST(sqlAST, context);
  if (!sql) return Promise.resolve({});

  return handleUserDbCall(dbCall, sql, shapeDefinition);
}

function compileSqlAST(sqlAST, context) {
  debug((0, _util.emphasize)('SQL_AST'), (0, _util.inspect)(sqlAST));

  const sql = (0, _stringifySqlAST2.default)(sqlAST, context);
  debug((0, _util.emphasize)('SQL'), (0, _util.inspect)(sql));

  const shapeDefinition = (0, _defineObjectShape2.default)(sqlAST);
  debug((0, _util.emphasize)('SHAPE_DEFINITION'), (0, _util.inspect)(shapeDefinition));
  return { sql, shapeDefinition };
}

function getNode(typeName, ast, context, where, dbCall) {
  const type = ast.schema.getType(typeName);

  const fakeParentNode = {
    _fields: {
      node: {
        type,
        name: type.name.toLowerCase(),
        where
      }
    }
  };
  const sqlAST = {};

  (0, _queryASTToSqlAST.getGraphQLType)(ast.fieldASTs[0], fakeParentNode, sqlAST, ast.fragments, new Set());
  const { sql, shapeDefinition } = compileSqlAST(sqlAST, context);
  return handleUserDbCall(dbCall, sql, shapeDefinition).then(obj => {
    obj.__type__ = type;
    return obj;
  });
}

joinMonster.getNode = getNode;

function handleUserDbCall(dbCall, sql, shapeDefinition) {
  if (dbCall.length === 2) {
    return new Promise((resolve, reject) => {
      dbCall(sql, (err, rows) => {
        if (err) {
          reject(err);
        } else {
          rows = validate(rows);
          debug((0, _util.emphasize)('RAW_DATA'), (0, _util.inspect)(rows.slice(0, 10)));
          debug(`${ rows.length } rows...`);
          resolve((0, _nesthydrationjs.nest)(rows, shapeDefinition));
        }
      });
    });
  }

  const result = dbCall(sql);

  if (typeof result.then === 'function') {
    return result.then(rows => {
      rows = validate(rows);
      debug((0, _util.emphasize)('RAW DATA'), (0, _util.inspect)(rows.slice(0, 10)));
      debug(`${ rows.length } rows...`);
      return (0, _nesthydrationjs.nest)(rows, shapeDefinition);
    });
  } else {
    return Promise.resolve((0, _nesthydrationjs.nest)(validate(result), shapeDefinition));
  }
}

function validate(rows) {
  if (Array.isArray(rows)) return rows;else if (rows.rows) return rows.rows;else {
      throw new Error(`"dbCall" function must return/resolve an array of objects where each object is a row from the result set. Instead got ${ _util3.default.inspect(rows, { depth: 3 }) }`);
    }
}

joinMonster.version = require('../package.json').version;
exports.default = joinMonster;